#include "map_iso.h"

Map_Iso::Map_Iso(SDL_Surface *_screen) {

	screen = _screen;

	cur_layer = "";
	cur_entity = -1;
	cur_trigger = -1;

	// cam(x,y) is where on the map the camera is pointing
	// units = 32
	cam.x = 0;
	cam.y = 0;
	
	this->load("map.txt");
}


/**
 * trim: remove leading and trailing c from s
 */
string trim(string s, char c) {
	int first = 0;
	int last = s.length()-1;
	if (last <= 0) return "";

	while (s.at(first) == c && first < s.length()-1) {
		first++;
	}
	while (s.at(last) == c && last >= first) {
		last--;
	}
	if (first <= last) return s.substr(first,last-first+1);
	return "";
}

string Map_Iso::parse_section_title(string s) {
	return s.substr(1, s.find_first_of(']') -1);
}

void Map_Iso::parse_key_pair(string s, string &key, string &val) {
	int colon = s.find_first_of(':');
	key = s.substr(0, colon);
	val = s.substr(colon+1, s.length());
}

/**
 * Given a string that starts with a number then a comma
 * Return that int, and modify the string to remove the num and comma
 *
 * This is basically a really lazy "split" replacement
 */
int Map_Iso::eatFirstInt(string &s, char separator) {
	int seppos = s.find_first_of(separator);
	int num = atoi(s.substr(0, seppos).c_str());
	s = s.substr(seppos+1, s.length());
	return num;
}


/**
 * load
 */
int Map_Iso::load(string filename) {
	ifstream infile;
	string line;
	string starts_with;
	string section;
	string key;
	string val;
  
	infile.open(filename.c_str(), ios::in);

	if (infile.is_open()) {
		while (!infile.eof()) {

			getline(infile, line);

			if (line.length() > 0) {
				starts_with = line.at(0);
				if (starts_with == "#") {
					// skip comments
				}
				else if (starts_with == "[") {
					section = trim(parse_section_title(line), ' ');
				}
				else { // this is data.  treatment depends on section type
					parse_key_pair(line, key, val);          
					key = trim(key, ' ');
					val = trim(val, ' ');

					if (section == "header") {
						if (key == "id") {
							this->id = val;
						}
						else if (key == "width") {
							this->w = atoi(val.c_str());
						}
						else if (key == "height") {
							this->h = atoi(val.c_str());
						}
						else if (key == "tileset") {
							this->tileset = val;
						}
						else if (key == "music") {
							this->music = val;
						}
						else if (key == "spawnpoint") {
							val = val + ",";
							spawn.x = eatFirstInt(val, ',');
							spawn.y = eatFirstInt(val, ',');
							spawn_dir = eatFirstInt(val, ',');
						}
					}
					else if (section == "layer") {
						if (key == "id") {
							cur_layer = val;
						}
						else if (key == "data") {
							// layer map data handled as a special case

							// The next h lines must contain layer data.  TODO: err
							for (int j=0; j<h; j++) {
								getline(infile, line);
								line = line + ',';
								for (int i=0; i<w; i++) {
									if (cur_layer == "background") background[i][j] = this->eatFirstInt(line, ',');
									else if (cur_layer == "fringe") fringe[i][j] = this->eatFirstInt(line, ',');
									else if (cur_layer == "object") object[i][j] = this->eatFirstInt(line, ',');
									else if (cur_layer == "collision") collision[i][j] = this->eatFirstInt(line, ',');
								}
							}
						}
					}
					else if (section == "entity") {
						if (key == "id") {
							cur_entity++;
							this->ent[cur_entity].id = val;
						}
						else if (key == "type") {

							// TODO: enum
							if (val == "enemy") this->ent[cur_entity].type = 1;
							else if (val == "npc") this->ent[cur_entity].type = 0;
						}
						else if (key == "location") {
							int comma = val.find_first_of(',');
							this->ent[cur_entity].x = atoi(val.substr(0,comma).c_str());
							this->ent[cur_entity].y = atoi(val.substr(comma+1,val.length()).c_str());
						}
           
					}
					else if (section == "trigger") {
						if (key == "id") {
							cur_trigger ++;
							this->trig[cur_trigger].id = val;
						}
					}
				}
			}
		}
	}
	infile.close();

	collider.setmap(collision);

	return 0;
}

void Map_Iso::render(Renderable r) {

	// r will become a list of renderables.  Everything not on the map already:
	// - hero
	// - npcs and monsters
	// - loot
	// maybe, special effects
	// we want to sort these by map draw order.  Then, we use a cursor to move through the 
	// renderables while we're also moving through the map tiles.  After we draw each map tile we
	// check to see if it's time to draw the next renderable yet.

	short unsigned int i;
	short unsigned int j;
	//SDL_Rect src;
	SDL_Rect dest;
	int current_tile;
	
	// todo: trim by screen rect
	// background
	for (j=0; j<h; j++) {
		for (i=0; i<w; i++) {
		  
			current_tile = background[i][j];
			
			if (current_tile > 0) {			
			
				dest.x = 320 + (i * 32 - cam.x) - (j * 32 - cam.y);
				dest.y = 240 + (i * 16 - (cam.x/2)) + (j * 16 - (cam.y/2));
				dest.x -= tset.tiles[current_tile].offset.x;
				dest.y -= tset.tiles[current_tile].offset.y;
				dest.w = tset.tiles[current_tile].src.w;
				dest.h = tset.tiles[current_tile].src.h;
				
				SDL_BlitSurface(tset.sprites, &(tset.tiles[current_tile].src), screen, &dest);
	
			}
		}
	}

	// todo: trim by screen rect
	// fringe
	for (j=0; j<h; j++) {
		for (i=0; i<w; i++) {
		  
			current_tile = fringe[i][j];
			
			if (current_tile > 0) {			
			
				dest.x = 320 + (i * 32 - cam.x) - (j * 32 - cam.y);
				dest.y = 240 + (i * 16 - (cam.x/2)) + (j * 16 - (cam.y/2));
				dest.x -= tset.tiles[current_tile].offset.x;
				dest.y -= tset.tiles[current_tile].offset.y;
				dest.w = tset.tiles[current_tile].src.w;
				dest.h = tset.tiles[current_tile].src.h;
				
				SDL_BlitSurface(tset.sprites, &(tset.tiles[current_tile].src), screen, &dest);
	
			}
		}
	}

	// todo: trim by screen rect
	// object layer
	for (j=0; j<h; j++) {
		for (i=0; i<w; i++) {
		
			current_tile = object[i][j];
			
			if (current_tile > 0) {			
			
				dest.x = 320 + (i * 32 - cam.x) - (j * 32 - cam.y);
				dest.y = 240 + (i * 16 - (cam.x/2)) + (j * 16 - (cam.y/2));
				dest.x -= tset.tiles[current_tile].offset.x;
				dest.y -= tset.tiles[current_tile].offset.y;
				dest.w = tset.tiles[current_tile].src.w;
				dest.h = tset.tiles[current_tile].src.h;
				
				SDL_BlitSurface(tset.sprites, &(tset.tiles[current_tile].src), screen, &dest);
	
			}
			
			// entities go in this layer			
			if (r.map_pos.x / 32 == i && r.map_pos.y / 32 == j) {
				
				// draw renderable
				dest.w = r.src->w;
				dest.h = r.src->h;
				dest.x = 320 - r.offset.x;
				dest.y = 240 - r.offset.y;
				SDL_BlitSurface(r.sprite, r.src, screen, &dest);
			}
						
		}
	}
	


	
}
